/* "multicap", multiple capture example for ESCAPI */
#include <windows.h>
#include <stdlib.h>
#include "SDL.h"
#include "escapi.h"

SDL_Surface *screen;
SDL_Surface *font;

struct SimpleCapParams capture[4];
int devices = 0;
char devicenames[4][24] = {"device 1","device 2","device 3","device 4"};

void drawchar(char ch, int x, int y)
{
  SDL_Rect src, dst;
  src.w = 14;
  src.h = 24;
  src.x = 0;
  src.y = (ch - 32) * 24;
  dst.w = 14;
  dst.h = 24;
  dst.x = x;
  dst.y = y;
  SDL_BlitSurface(font, &src, screen, &dst);
}

void drawstring(char * string, int x, int y)
{
  while (*string)
  {
    drawchar(*string,x,y);
    x += 14;
    string++;
  }
}


void render()
{
  // Declare a couple of variables
  int i, j, k;

  for (k = 0; k < devices; k++)
  {
    if (isCaptureDone(k))
    {
      int linofs = 0;
      int xofs = (k&1) ? 320 : 0;
      int yofs = (k&2) ? (240 * screen->pitch / 4) : 0;
      int ofs;

      // Draw to screen
      for (i = 0; i < 240; i++)
      {
        for (j = 0, ofs = yofs + xofs; j < 320; j++, ofs++, linofs++)
        {
          ((unsigned int*)screen->pixels)[ofs] = capture[k].mTargetBuf[linofs];
        }
        yofs += screen->pitch / 4;
      }
      doCapture(k);

      drawstring(devicenames[k],xofs,(k&2)?240:0);
    }
  }

  Sleep(1);

  // Tell SDL to update the whole screen
  SDL_UpdateRect(screen, 0, 0, 640, 480);
}


// Entry point
int main(int argc, char *argv[])
{
  // Initialize SDL's subsystems - in this case, only video.
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
  {
    fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
    exit(1);
  }

  // Register SDL_Quit to be called at exit; makes sure things are
  // cleaned up when we quit.
  atexit(SDL_Quit);

  devices = setupESCAPI();
  if (devices < 1)
  {
    fprintf(stderr, "Unable to init ESCAPI\n");
    exit(1);
  }
  
  if (devices > 4)
    devices = 4;

  for (int i = 0; i < devices; i++)
  {
    capture[i].mWidth = 320;
    capture[i].mHeight = 240;
    capture[i].mTargetBuf = new int[320*240];
    getCaptureDeviceName(i,devicenames[i],24);
    initCapture(i,&capture[i]);
    doCapture(i);
  }

  // Attempt to create a 640x480 window with 32bit pixels.
  screen = SDL_SetVideoMode(640, 480, 32, SDL_SWSURFACE);

  // If we fail, return error.
  if ( screen == NULL )
  {
    fprintf(stderr, "Unable to set 640x480 video: %s\n", SDL_GetError());
    exit(1);
  }

  // load font
  SDL_Surface *temp = SDL_LoadBMP("font14x24.bmp");
  font = SDL_ConvertSurface(temp, screen->format, SDL_SWSURFACE);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(font, SDL_SRCCOLORKEY, 0);

  // Main loop: loop forever.
  while (1)
  {
    // Render stuff
    render();

    // Poll for events, and handle the ones we care about.
    SDL_Event event;
    while (SDL_PollEvent(&event))
    {
      switch (event.type)
      {
      case SDL_KEYDOWN:
        break;
      case SDL_KEYUP:
        // If escape is pressed, return (and thus, quit)
        if (event.key.keysym.sym == SDLK_ESCAPE)
        {
          int k;
          for (k = 0; k < devices; k++)
            deinitCapture(k);
          return 0;
        }
        break;
      case SDL_QUIT:
        return(0);
      }
    }
  }
  return 0;
}

